# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pdk8s',
 'pdk8s.cookiecutter.{{cookiecutter.slug}}',
 'pdk8s.gen',
 'pdk8s.gen.io',
 'pdk8s.gen.io.k8s',
 'pdk8s.gen.io.k8s.api',
 'pdk8s.gen.io.k8s.api.admissionregistration',
 'pdk8s.gen.io.k8s.api.apps',
 'pdk8s.gen.io.k8s.api.auditregistration',
 'pdk8s.gen.io.k8s.api.authentication',
 'pdk8s.gen.io.k8s.api.authorization',
 'pdk8s.gen.io.k8s.api.autoscaling',
 'pdk8s.gen.io.k8s.api.batch',
 'pdk8s.gen.io.k8s.api.certificates',
 'pdk8s.gen.io.k8s.api.coordination',
 'pdk8s.gen.io.k8s.api.core',
 'pdk8s.gen.io.k8s.api.discovery',
 'pdk8s.gen.io.k8s.api.events',
 'pdk8s.gen.io.k8s.api.extensions',
 'pdk8s.gen.io.k8s.api.networking',
 'pdk8s.gen.io.k8s.api.node',
 'pdk8s.gen.io.k8s.api.policy',
 'pdk8s.gen.io.k8s.api.rbac',
 'pdk8s.gen.io.k8s.api.scheduling',
 'pdk8s.gen.io.k8s.api.settings',
 'pdk8s.gen.io.k8s.api.storage',
 'pdk8s.gen.io.k8s.apiextensions_apiserver',
 'pdk8s.gen.io.k8s.apiextensions_apiserver.pkg',
 'pdk8s.gen.io.k8s.apiextensions_apiserver.pkg.apis',
 'pdk8s.gen.io.k8s.apiextensions_apiserver.pkg.apis.apiextensions',
 'pdk8s.gen.io.k8s.apimachinery',
 'pdk8s.gen.io.k8s.apimachinery.pkg',
 'pdk8s.gen.io.k8s.apimachinery.pkg.api',
 'pdk8s.gen.io.k8s.apimachinery.pkg.apis',
 'pdk8s.gen.io.k8s.apimachinery.pkg.apis.meta',
 'pdk8s.gen.io.k8s.apimachinery.pkg.util',
 'pdk8s.gen.io.k8s.kube_aggregator',
 'pdk8s.gen.io.k8s.kube_aggregator.pkg',
 'pdk8s.gen.io.k8s.kube_aggregator.pkg.apis',
 'pdk8s.gen.io.k8s.kube_aggregator.pkg.apis.apiregistration']

package_data = \
{'': ['*'], 'pdk8s': ['cookiecutter/*']}

install_requires = \
['click==7.1.2',
 'cookiecutter==1.7.2',
 'datamodel-code-generator>=0.5.9,<0.6.0',
 'pydantic==1.5.1',
 'pyyaml==5.3.1',
 'setuptools==47.1.1']

entry_points = \
{'console_scripts': ['pdk8s = pdk8s.cli:main']}

setup_kwargs = {
    'name': 'pdk8s',
    'version': '0.2.2',
    'description': '',
    'long_description': '![CI](https://github.com/FlorianLudwig/pdk8s/workflows/CI/badge.svg)\n![license](https://img.shields.io/github/license/FlorianLudwig/pdk8s.svg)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n# pdk8s\n\nGenerating Kubernetes definitions (yaml) with python, inspired by [cdk8s](https://github.com/awslabs/cdk8s).  The main use case is to use those definitions with helm.  This means cdk8s does replace all the templating that helm does - but helm still takes care of rolling out your changes to your cluster.\n\n\n# Getting started\n\n<!-- add nice video here -->\n\n## Installing pdk8s\n\n### Prerequisites\n\n* Python >= 3.7.\n* Python knowledge\n\n\n### Installation via PyPi\n\npdk8s is available on PyPi, you can install it with your preferred python package manage, `pip`, `pipenv`, etc:\n\n```\npip install pdk8s\n```\n\n## Intro\n\nThe format of `pdk8s` charts is similar to helm charts, just that they are python instead of yaml.  Your “python chart" must define the following variables:\n\n * `name`: Name of your chart\n * `chart_version`: This is the chart version. This version number should be incremented each time you make changes to the chart and its templates, including the app version. Versions are expected to follow [Semantic Versioning](https://semver.org/).\n * `app_version`: This is the version number of the application being deployed. This version number should be incremented each time you make changes to the application. Versions are not expected to follow Semantic Versioning. They should reflect the version the application is using.\n * `chart`: Your chart. A list or `pdk8s.k8s.Chart` (or actually any iterable python object) of k8s resources.\n\nIf you had a déjà vu while reading - that is because the description for `chart_version` and `app_version` are copied straight from Helm ;)\n\n## Getting started\n\n```bash\n$ pdk8s init\nchart_name [awesome chart]: Webserver Example\nslug [webserver_example]: \nchart_version [0.1.0]: \napp_version [0.1.0]: \n```\n\nYou will find a new folder and files named: `webserver_example/chart.py`.  Inside this file you will find a hello world example:\n\n```python\nchart = [\n    k8s.Deployment(name=\'deployment\',\n                    spec=k8s.DeploymentSpec(\n                        replicas=2,\n                        selector=k8s.LabelSelector(match_labels=label),\n                        template=k8s.PodTemplateSpec(\n                        metadata=k8s.ObjectMeta(labels=label),\n                        spec=k8s.PodSpec(containers=[\n                            k8s.Container(\n                            name=\'hello-kubernetes\',\n                            image=\'paulbouwer/hello-kubernetes:1.7\',\n                            ports=[k8s.ContainerPort(container_port=8080)])]))))\n]\n```\n\nWhich you can turn into a running helm chart with:\n\n```bash\n$ pdk8s synth\n```\n\nYou will find your generated chart under `dist`:\n\n<pre>\n├── chart.py\n└── <font color="#0087FF">dist</font>\n    ├── Chart.yaml\n    ├── <font color="#0087FF">templates</font>\n    │\xa0\xa0 └── generated.yaml\n    └── values.yaml\n</pre>\n\nPer default `pdk8s synth` loads the `chart.py` in the current directory.  You can also use `-i` to specify a different python file.  Also the `chart.py` generated by `pdk8s init` provides the same api as `pdk8s` except without the `-i` option:\n\n```bash\n$ ./chart.py synth\n```\n\n## Creating Ressources\n\nCreating a service:\n\n```python\nfrom pdk8s import k8s\n\nservice = k8s.Service(name="service",\n            spec=k8s.ServiceSpec(\n                type="LoadBalancer",\n                ports=[k8s.ServicePort(port=80, target_port=8080)],\n                selector={"app": "hello-k8s"}))\n\nchart = [service]\n```\n\nAll `pdk8s` classes are [pydantic](https://pydantic-docs.helpmanual.io/) data classes.  Which provides - among other things - automatic conversion for parameters, so you can just as well write:\n\n```python\nfrom pdk8s import k8s\n\nk8s.Service(name="service",\n            spec={\n                "type": "LoadBalancer",\n                "ports": [{"port": 80, "target_port": 8080}],\n                "selector": {"app": "hello-k8s"}})\n```\n\n## Manipulating\n\nAll attributes can be manipulated after creation:\n\n```python\n\ndeployment = k8s.Deployment(name=\'deployment\',\n                    spec=k8s.DeploymentSpec(\n                        replicas=2))\n\ndeployment.spec.replicas = math.randint(0, 666)\n```\n\nNote 1: Automatic casting is only available on creation.  `deployment.spec = {"replicas": 2}` would not work.\n\nNote 2: Currently all required parameters must be provided at creation time.  You cannot create an empty `k8s.Deployment()`.  This might change.\n\n\n## CamelCase names\n\nThe Kubernetes APIs use camelCase for naming attributes, while python usually uses snake_case.  `pdk8s` also follows the snake_case convention, same as `cdk8s`.\n\n`pdk8s` provides aliases for all arguments:\n\n```python\nk8s.ServicePort(port=80, target_port=8080)\nk8s.ServicePort(port=80, targetPort=8080)\n```\n\nBoth work and result in the same result.  This is for compatibility when importing from other sources (and makes `pdk8s.k8s.parse` possible).\n\n## Importing existing charts\n\nYou might already have templates you want to build upon, you can easily import them using `pdk8s.k8s.parse`. Let\'s assume you have the following `chart.yaml`:\n\n```yaml\napiVersion: v1\nkind: Service\nmetadata:\n  name: service\nspec:\n  ports:\n  - port: 80\n    targetPort: 8080\n  selector:\n    app: hello-k8s\n  type: LoadBalancer\n```\n\nWith:\n\n```python\nimport pdk8s\nfrom pdk8s import k8s\n\nmy_chart = k8s.parse("example/chart.yaml")\nmy_chart[0].name = "service_new"\npdk8s.synth(my_chart)\n```\n\nYou get:\n\n```yaml\napiVersion: v1\nkind: Service\nmetadata:\n  name: service_new\nspec:\n  ports:\n  - port: 80\n    targetPort: 8080\n  selector:\n    app: hello-k8s\n  type: LoadBalancer\n```\n\n# Compatibility to cdk8s\n\nThere are a few differences that make code between the `cdk8s` and `pdk8s` incompatible.  A good overview can be archived by comparing the following to examples:\n\n * [cdk8s example](https://github.com/awslabs/cdk8s/blob/master/docs/getting-started/python.md#importing-constructs-for-the-kubernetes-api)\n * [pdk8s example](https://github.com/FlorianLudwig/pdk8s/blob/master/example/hello_world.py)\n\n### Pure python\n\n`cdk8s` is written in TypeScript and with the power of jsii usable from other languages, as python.  `pdk8s` is written in pure python with no bridge to other languages.  This means you are limited to python and cannot reuse charts written in other languages.  Therefore, a `pdk8s` is focused on providing an awesome experience writing charts in python: Readable tracebacks, happy IDE and linters, ... \n\n### Context / Constructs\n\nCurrently, there is no equivalent of "constructs" in `pdk8s`.  In `cdk8s` highlevel objects (e.g. `Service`) are special: They have an extra argument (the first one) which is the context in which they are defined, e.g. `k8s.Service(self, ...)` where `self` is the context.\n\nIn `pdk8s` there is no special treatment of these types.  There might be later on, but they would be added and not replaced.\n\nThis allows for more flexibility on how to construct your chart generator.\n\n\n### Names\n\nIn `cdk8s` names are automatically made unique by adding a hash to it.  `pdk8s` does not observe this behavior.  Also in `pdk8s` names must be provided as keyword argument.\n\n```python\n\n# cdk8s\nk8s.Service(Chart("hello"), "service")\n# kind: Service\n# apiVersion: v1\n# metadata:\n#   name: hello-service-9878228b\n\n\n# pdk8s\nk8s.Service(name=\'service\')\n# kind: Service\n# apiVersion: v1\n# metadata:\n#   name: service\n\n\n```\n<!-- TODO add reasoning -->\n\n### IntOrString\n\n```python\n# cdk8s\nk8s.ServicePort(port=80, target_port=k8s.IntOrString.from_number(8080))\n\n# pdk8s\nk8s.ServicePort(port=80, target_port=8080)\n# k8s.IntOrString might be added for compatibility later on\n```\n\n\n# Why\n\nTODO explain why this exists (NIH syndrom)\n## Design Decisions\n\n### Generate at build time\n\nGenerate everything at build time and not runtime as it makes it easier for linters and other dev tools, like IDEs.\n\n### Attribute case\n\nCamelCase?\n\n### Naming\n\n\n### Versioning\n\n# Development and building\n\nCurrently, generating the code of `pdk8s` depends on a patched version of `datamodel-code-generator`.  I am working on upstreaming changes to not depend on local patches anymore.\n\n## Sources\n\n * https://github.com/kubernetes/kubernetes/tree/master/api/openapi-spec - openapi definition.\n * https://github.com/instrumenta/kubernetes-json-schema - JSON Schema of Kubernetes API, generated from official openapi definitions.  Used by cdk8s.\n',
    'author': 'Florian Ludwig',
    'author_email': 'f.ludwig@greyrook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from pydantic import BaseModel


class Info(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    build_date: str = Field(..., alias="buildDate")
    compiler: str
    git_commit: str = Field(..., alias="gitCommit")
    git_tree_state: str = Field(..., alias="gitTreeState")
    git_version: str = Field(..., alias="gitVersion")
    go_version: str = Field(..., alias="goVersion")
    major: str
    minor: str
    platform: str



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind126, Kind127
from ...apimachinery.pkg.apis.meta import v1
from ..core import v1


class Overhead(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    pod_fixed: Optional[Dict[str, Any]] = Field(
        None,
        alias="podFixed",
        description="PodFixed represents the fixed resource overhead associated with running a pod.",
    )


class Scheduling(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    node_selector: Optional[Dict[str, Any]] = Field(
        None,
        alias="nodeSelector",
        description="nodeSelector lists labels that must be present on nodes that support this RuntimeClass. Pods using this RuntimeClass can only be scheduled to a node matched by this selector. The RuntimeClass nodeSelector is merged with a pod's existing nodeSelector. Any conflicts will cause the pod to be rejected in admission.",
    )
    tolerations: Optional[List[v1.Toleration]] = Field(
        None,
        description="tolerations are appended (excluding duplicates) to pods running with this RuntimeClass during admission, effectively unioning the set of nodes tolerated by the pod and the RuntimeClass.",
    )


class RuntimeClass(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "node.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    handler: str = Field(
        ...,
        description='Handler specifies the underlying runtime and configuration that the CRI implementation will use to handle pods of this class. The possible values are specific to the node & CRI configuration.  It is assumed that all handlers are available on every node, and handlers of the same name are equivalent on every node. For example, a handler called "runc" might specify that the runc OCI runtime (using native Linux containers) will be used to run the containers in a pod. The Handler must conform to the DNS Label (RFC 1123) requirements, and is immutable.',
    )
    kind: Optional[Kind126] = Field(
        "RuntimeClass",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    overhead: Optional[Overhead] = Field(
        None,
        description="Overhead represents the resource overhead associated with running a pod for a given RuntimeClass. For more details, see https://git.k8s.io/enhancements/keps/sig-node/20190226-pod-overhead.md This field is alpha-level as of Kubernetes v1.15, and is only honored by servers that enable the PodOverhead feature.",
    )
    scheduling: Optional[Scheduling] = Field(
        None,
        description="Scheduling holds the scheduling constraints to ensure that pods running with this RuntimeClass are scheduled to nodes that support it. If scheduling is nil, this RuntimeClass is assumed to be supported by all nodes.",
    )


class RuntimeClassList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "node.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[RuntimeClass] = Field(
        ..., description="Items is a list of schema objects."
    )
    kind: Optional[Kind127] = Field(
        "RuntimeClassList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind104, Kind105
from ...apimachinery.pkg.apis.meta import v1
from ..core import v1 as v1_1


class EventSeries(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    count: int = Field(
        ...,
        description="Number of occurrences in this series up to the last heartbeat time",
    )
    last_observed_time: v1.MicroTime = Field(
        ...,
        alias="lastObservedTime",
        description="Time when last Event from the series was seen before last heartbeat.",
    )
    state: str = Field(
        ...,
        description="Information whether this series is ongoing or finished. Deprecated. Planned removal for 1.18",
    )


class Event(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    action: Optional[str] = Field(
        None,
        description="What action was taken/failed regarding to the regarding object.",
    )
    api_version: Optional[str] = Field(
        "events.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    deprecated_count: Optional[int] = Field(
        None,
        alias="deprecatedCount",
        description="Deprecated field assuring backward compatibility with core.v1 Event type",
    )
    deprecated_first_timestamp: Optional[v1.Time] = Field(
        None,
        alias="deprecatedFirstTimestamp",
        description="Deprecated field assuring backward compatibility with core.v1 Event type",
    )
    deprecated_last_timestamp: Optional[v1.Time] = Field(
        None,
        alias="deprecatedLastTimestamp",
        description="Deprecated field assuring backward compatibility with core.v1 Event type",
    )
    deprecated_source: Optional[v1_1.EventSource] = Field(
        None,
        alias="deprecatedSource",
        description="Deprecated field assuring backward compatibility with core.v1 Event type",
    )
    event_time: v1.MicroTime = Field(
        ...,
        alias="eventTime",
        description="Required. Time when this Event was first observed.",
    )
    kind: Optional[Kind104] = Field(
        "Event",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    note: Optional[str] = Field(
        None,
        description="Optional. A human-readable description of the status of this operation. Maximal length of the note is 1kB, but libraries should be prepared to handle values up to 64kB.",
    )
    reason: Optional[str] = Field(None, description="Why the action was taken.")
    regarding: Optional[v1_1.ObjectReference] = Field(
        None,
        description="The object this Event is about. In most cases it's an Object reporting controller implements. E.g. ReplicaSetController implements ReplicaSets and this event is emitted because it acts on some changes in a ReplicaSet object.",
    )
    related: Optional[v1_1.ObjectReference] = Field(
        None,
        description="Optional secondary object for more complex actions. E.g. when regarding object triggers a creation or deletion of related object.",
    )
    reporting_controller: Optional[str] = Field(
        None,
        alias="reportingController",
        description="Name of the controller that emitted this Event, e.g. `kubernetes.io/kubelet`.",
    )
    reporting_instance: Optional[str] = Field(
        None,
        alias="reportingInstance",
        description="ID of the controller instance, e.g. `kubelet-xyzf`.",
    )
    series: Optional[EventSeries] = Field(
        None,
        description="Data about the Event series this event represents or nil if it's a singleton Event.",
    )
    type: Optional[str] = Field(
        None,
        description="Type of this event (Normal, Warning), new types could be added in the future.",
    )


class EventList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "events.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Event] = Field(..., description="Items is a list of schema objects.")
    kind: Optional[Kind105] = Field(
        "EventList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind42, Kind43, Kind44, Kind45
from ...apimachinery.pkg.apis.meta import v1


class NonResourceAttributes(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    path: Optional[str] = Field(None, description="Path is the URL path of the request")
    verb: Optional[str] = Field(None, description="Verb is the standard HTTP verb")


class NonResourceRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    non_resource_ur_ls: Optional[List[str]] = Field(
        None,
        alias="nonResourceURLs",
        description='NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path.  "*" means all.',
    )
    verbs: List[str] = Field(
        ...,
        description='Verb is a list of kubernetes non-resource API verbs, like: get, post, put, delete, patch, head, options.  "*" means all.',
    )


class ResourceAttributes(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    group: Optional[str] = Field(
        None, description='Group is the API Group of the Resource.  "*" means all.'
    )
    name: Optional[str] = Field(
        None,
        description='Name is the name of the resource being requested for a "get" or deleted for a "delete". "" (empty) means all.',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace is the namespace of the action being requested.  Currently, there is no distinction between no namespace and all namespaces "" (empty) is defaulted for LocalSubjectAccessReviews "" (empty) is empty for cluster-scoped resources "" (empty) means "all" for namespace scoped resources from a SubjectAccessReview or SelfSubjectAccessReview',
    )
    resource: Optional[str] = Field(
        None,
        description='Resource is one of the existing resource types.  "*" means all.',
    )
    subresource: Optional[str] = Field(
        None,
        description='Subresource is one of the existing resource types.  "" means none.',
    )
    verb: Optional[str] = Field(
        None,
        description='Verb is a kubernetes resource API verb, like: get, list, watch, create, update, delete, proxy.  "*" means all.',
    )
    version: Optional[str] = Field(
        None, description='Version is the API Version of the Resource.  "*" means all.'
    )


class ResourceRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_groups: Optional[List[str]] = Field(
        None,
        alias="apiGroups",
        description='APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  "*" means all.',
    )
    resource_names: Optional[List[str]] = Field(
        None,
        alias="resourceNames",
        description='ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  "*" means all.',
    )
    resources: Optional[List[str]] = Field(
        None,
        description='Resources is a list of resources this rule applies to.  "*" means all in the specified apiGroups.\n "*/foo" represents the subresource \'foo\' for all resources in the specified apiGroups.',
    )
    verbs: List[str] = Field(
        ...,
        description='Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  "*" means all.',
    )


class SelfSubjectAccessReviewSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    non_resource_attributes: Optional[NonResourceAttributes] = Field(
        None,
        alias="nonResourceAttributes",
        description="NonResourceAttributes describes information for a non-resource access request",
    )
    resource_attributes: Optional[ResourceAttributes] = Field(
        None,
        alias="resourceAttributes",
        description="ResourceAuthorizationAttributes describes information for a resource access request",
    )


class SelfSubjectRulesReviewSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    namespace: Optional[str] = Field(
        None, description="Namespace to evaluate rules for. Required."
    )


class SubjectAccessReviewSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    extra: Optional[Dict[str, Any]] = Field(
        None,
        description="Extra corresponds to the user.Info.GetExtra() method from the authenticator.  Since that is input to the authorizer it needs a reflection here.",
    )
    groups: Optional[List[str]] = Field(
        None, description="Groups is the groups you're testing for."
    )
    non_resource_attributes: Optional[NonResourceAttributes] = Field(
        None,
        alias="nonResourceAttributes",
        description="NonResourceAttributes describes information for a non-resource access request",
    )
    resource_attributes: Optional[ResourceAttributes] = Field(
        None,
        alias="resourceAttributes",
        description="ResourceAuthorizationAttributes describes information for a resource access request",
    )
    uid: Optional[str] = Field(
        None, description="UID information about the requesting user."
    )
    user: Optional[str] = Field(
        None,
        description='User is the user you\'re testing for. If you specify "User" but not "Groups", then is it interpreted as "What if User were not a member of any groups',
    )


class SubjectAccessReviewStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    allowed: bool = Field(
        ...,
        description="Allowed is required. True if the action would be allowed, false otherwise.",
    )
    denied: Optional[bool] = Field(
        None,
        description="Denied is optional. True if the action would be denied, otherwise false. If both allowed is false and denied is false, then the authorizer has no opinion on whether to authorize the action. Denied may not be true if Allowed is true.",
    )
    evaluation_error: Optional[str] = Field(
        None,
        alias="evaluationError",
        description="EvaluationError is an indication that some error occurred during the authorization check. It is entirely possible to get an error and be able to continue determine authorization status in spite of it. For instance, RBAC can be missing a role, but enough roles are still present and bound to reason about the request.",
    )
    reason: Optional[str] = Field(
        None,
        description="Reason is optional.  It indicates why a request was allowed or denied.",
    )


class SubjectRulesReviewStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    evaluation_error: Optional[str] = Field(
        None,
        alias="evaluationError",
        description="EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.",
    )
    incomplete: bool = Field(
        ...,
        description="Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.",
    )
    non_resource_rules: List[NonResourceRule] = Field(
        ...,
        alias="nonResourceRules",
        description="NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
    )
    resource_rules: List[ResourceRule] = Field(
        ...,
        alias="resourceRules",
        description="ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
    )


class LocalSubjectAccessReview(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind42] = Field(
        "LocalSubjectAccessReview",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: SubjectAccessReviewSpec = Field(
        ...,
        description="Spec holds information about the request being evaluated.  spec.namespace must be equal to the namespace you made the request against.  If empty, it is defaulted.",
    )


class SelfSubjectAccessReview(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind43] = Field(
        "SelfSubjectAccessReview",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: SelfSubjectAccessReviewSpec = Field(
        ...,
        description="Spec holds information about the request being evaluated.  user and groups must be empty",
    )


class SelfSubjectRulesReview(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind44] = Field(
        "SelfSubjectRulesReview",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: SelfSubjectRulesReviewSpec = Field(
        ..., description="Spec holds information about the request being evaluated."
    )


class SubjectAccessReview(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "authorization.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind45] = Field(
        "SubjectAccessReview",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: SubjectAccessReviewSpec = Field(
        ..., description="Spec holds information about the request being evaluated"
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind57, Kind58
from ...apimachinery.pkg.apis.meta import v1
from ..core import v1 as v1_1


class JobCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_probe_time: Optional[v1.Time] = Field(
        None, alias="lastProbeTime", description="Last time the condition was checked."
    )
    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transit from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="Human readable message indicating details about last transition.",
    )
    reason: Optional[str] = Field(
        None, description="(brief) reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of job condition, Complete or Failed.")


class JobStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    active: Optional[int] = Field(
        None, description="The number of actively running pods."
    )
    completion_time: Optional[v1.Time] = Field(
        None,
        alias="completionTime",
        description="Represents time when the job was completed. It is not guaranteed to be set in happens-before order across separate operations. It is represented in RFC3339 form and is in UTC.",
    )
    conditions: Optional[List[JobCondition]] = Field(
        None,
        description="The latest available observations of an object's current state. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
    )
    failed: Optional[int] = Field(
        None, description="The number of pods which reached phase Failed."
    )
    start_time: Optional[v1.Time] = Field(
        None,
        alias="startTime",
        description="Represents time when the job was acknowledged by the job controller. It is not guaranteed to be set in happens-before order across separate operations. It is represented in RFC3339 form and is in UTC.",
    )
    succeeded: Optional[int] = Field(
        None, description="The number of pods which reached phase Succeeded."
    )


class JobSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    active_deadline_seconds: Optional[int] = Field(
        None,
        alias="activeDeadlineSeconds",
        description="Specifies the duration in seconds relative to the startTime that the job may be active before the system tries to terminate it; value must be positive integer",
    )
    backoff_limit: Optional[int] = Field(
        None,
        alias="backoffLimit",
        description="Specifies the number of retries before marking this job failed. Defaults to 6",
    )
    completions: Optional[int] = Field(
        None,
        description="Specifies the desired number of successfully finished pods the job should be run with.  Setting to nil means that the success of any pod signals the success of all pods, and allows parallelism to have any positive value.  Setting to 1 means that parallelism is limited to 1 and the success of that pod signals the success of the job. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
    )
    manual_selector: Optional[bool] = Field(
        None,
        alias="manualSelector",
        description="manualSelector controls generation of pod labels and pod selectors. Leave `manualSelector` unset unless you are certain what you are doing. When false or unset, the system pick labels unique to this job and appends those labels to the pod template.  When true, the user is responsible for picking unique labels and specifying the selector.  Failure to pick a unique label may cause this and other jobs to not function correctly.  However, You may see `manualSelector=true` in jobs that were created with the old `extensions/v1beta1` API. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/#specifying-your-own-pod-selector",
    )
    parallelism: Optional[int] = Field(
        None,
        description="Specifies the maximum desired number of pods the job should run at any given time. The actual number of pods running in steady state will be less than this number when ((.spec.completions - .status.successful) < .spec.parallelism), i.e. when the work left to do is less than max parallelism. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="A label query over pods that should match the pod count. Normally, the system sets this field for you. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )
    template: v1_1.PodTemplateSpec = Field(
        ...,
        description="Describes the pod that will be created when executing a job. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/",
    )
    ttl_seconds_after_finished: Optional[int] = Field(
        None,
        alias="ttlSecondsAfterFinished",
        description="ttlSecondsAfterFinished limits the lifetime of a Job that has finished execution (either Complete or Failed). If this field is set, ttlSecondsAfterFinished after the Job finishes, it is eligible to be automatically deleted. When the Job is being deleted, its lifecycle guarantees (e.g. finalizers) will be honored. If this field is unset, the Job won't be automatically deleted. If this field is set to zero, the Job becomes eligible to be deleted immediately after it finishes. This field is alpha-level and is only honored by servers that enable the TTLAfterFinished feature.",
    )


class Job(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "batch/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind57] = Field(
        "Job",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[JobSpec] = Field(
        None,
        description="Specification of the desired behavior of a job. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class JobList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "batch/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Job] = Field(..., description="items is the list of Jobs.")
    kind: Optional[Kind58] = Field(
        "JobList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind53, Kind54
from ...apimachinery.pkg.api import resource
from ...apimachinery.pkg.apis.meta import v1


class CrossVersionObjectReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        None, alias="apiVersion", description="API version of the referent"
    )
    kind: str = Field(
        ...,
        description='Kind of the referent; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"',
    )
    name: str = Field(
        ...,
        description="Name of the referent; More info: http://kubernetes.io/docs/user-guide/identifiers#names",
    )


class HorizontalPodAutoscalerCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="lastTransitionTime is the last time the condition transitioned from one status to another",
    )
    message: Optional[str] = Field(
        None,
        description="message is a human-readable explanation containing details about the transition",
    )
    reason: Optional[str] = Field(
        None, description="reason is the reason for the condition's last transition."
    )
    type: str = Field(..., description="type describes the current condition")


class ResourceMetricSource(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: str = Field(..., description="name is the name of the resource in question.")
    target_average_utilization: Optional[int] = Field(
        None,
        alias="targetAverageUtilization",
        description="targetAverageUtilization is the target value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.",
    )
    target_average_value: Optional[resource.Quantity] = Field(
        None,
        alias="targetAverageValue",
        description='targetAverageValue is the target value of the average of the resource metric across all relevant pods, as a raw value (instead of as a percentage of the request), similar to the "pods" metric source type.',
    )


class ResourceMetricStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    current_average_utilization: Optional[int] = Field(
        None,
        alias="currentAverageUtilization",
        description="currentAverageUtilization is the current value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.  It will only be present if `targetAverageValue` was set in the corresponding metric specification.",
    )
    current_average_value: resource.Quantity = Field(
        ...,
        alias="currentAverageValue",
        description='currentAverageValue is the current value of the average of the resource metric across all relevant pods, as a raw value (instead of as a percentage of the request), similar to the "pods" metric source type. It will always be set, regardless of the corresponding metric specification.',
    )
    name: str = Field(..., description="name is the name of the resource in question.")


class ExternalMetricSource(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    metric_name: str = Field(
        ...,
        alias="metricName",
        description="metricName is the name of the metric in question.",
    )
    metric_selector: Optional[v1.LabelSelector] = Field(
        None,
        alias="metricSelector",
        description="metricSelector is used to identify a specific time series within a given metric.",
    )
    target_average_value: Optional[resource.Quantity] = Field(
        None,
        alias="targetAverageValue",
        description="targetAverageValue is the target per-pod value of global metric (as a quantity). Mutually exclusive with TargetValue.",
    )
    target_value: Optional[resource.Quantity] = Field(
        None,
        alias="targetValue",
        description="targetValue is the target value of the metric (as a quantity). Mutually exclusive with TargetAverageValue.",
    )


class ExternalMetricStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    current_average_value: Optional[resource.Quantity] = Field(
        None,
        alias="currentAverageValue",
        description="currentAverageValue is the current value of metric averaged over autoscaled pods.",
    )
    current_value: resource.Quantity = Field(
        ...,
        alias="currentValue",
        description="currentValue is the current value of the metric (as a quantity)",
    )
    metric_name: str = Field(
        ...,
        alias="metricName",
        description="metricName is the name of a metric used for autoscaling in metric system.",
    )
    metric_selector: Optional[v1.LabelSelector] = Field(
        None,
        alias="metricSelector",
        description="metricSelector is used to identify a specific time series within a given metric.",
    )


class ObjectMetricSource(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    average_value: Optional[resource.Quantity] = Field(
        None,
        alias="averageValue",
        description="averageValue is the target value of the average of the metric across all relevant pods (as a quantity)",
    )
    metric_name: str = Field(
        ...,
        alias="metricName",
        description="metricName is the name of the metric in question.",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="selector is the string-encoded form of a standard kubernetes label selector for the given metric When set, it is passed as an additional parameter to the metrics server for more specific metrics scoping When unset, just the metricName will be used to gather metrics.",
    )
    target: CrossVersionObjectReference = Field(
        ..., description="target is the described Kubernetes object."
    )
    target_value: resource.Quantity = Field(
        ...,
        alias="targetValue",
        description="targetValue is the target value of the metric (as a quantity).",
    )


class ObjectMetricStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    average_value: Optional[resource.Quantity] = Field(
        None,
        alias="averageValue",
        description="averageValue is the current value of the average of the metric across all relevant pods (as a quantity)",
    )
    current_value: resource.Quantity = Field(
        ...,
        alias="currentValue",
        description="currentValue is the current value of the metric (as a quantity).",
    )
    metric_name: str = Field(
        ...,
        alias="metricName",
        description="metricName is the name of the metric in question.",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="selector is the string-encoded form of a standard kubernetes label selector for the given metric When set in the ObjectMetricSource, it is passed as an additional parameter to the metrics server for more specific metrics scoping. When unset, just the metricName will be used to gather metrics.",
    )
    target: CrossVersionObjectReference = Field(
        ..., description="target is the described Kubernetes object."
    )


class PodsMetricSource(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    metric_name: str = Field(
        ...,
        alias="metricName",
        description="metricName is the name of the metric in question",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="selector is the string-encoded form of a standard kubernetes label selector for the given metric When set, it is passed as an additional parameter to the metrics server for more specific metrics scoping When unset, just the metricName will be used to gather metrics.",
    )
    target_average_value: resource.Quantity = Field(
        ...,
        alias="targetAverageValue",
        description="targetAverageValue is the target value of the average of the metric across all relevant pods (as a quantity)",
    )


class PodsMetricStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    current_average_value: resource.Quantity = Field(
        ...,
        alias="currentAverageValue",
        description="currentAverageValue is the current value of the average of the metric across all relevant pods (as a quantity)",
    )
    metric_name: str = Field(
        ...,
        alias="metricName",
        description="metricName is the name of the metric in question",
    )
    selector: Optional[v1.LabelSelector] = Field(
        None,
        description="selector is the string-encoded form of a standard kubernetes label selector for the given metric When set in the PodsMetricSource, it is passed as an additional parameter to the metrics server for more specific metrics scoping. When unset, just the metricName will be used to gather metrics.",
    )


class MetricSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    external: Optional[ExternalMetricSource] = Field(
        None,
        description="external refers to a global metric that is not associated with any Kubernetes object. It allows autoscaling based on information coming from components running outside of cluster (for example length of queue in cloud messaging service, or QPS from loadbalancer running outside of cluster).",
    )
    object: Optional[ObjectMetricSource] = Field(
        None,
        description="object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object).",
    )
    pods: Optional[PodsMetricSource] = Field(
        None,
        description="pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value.",
    )
    resource: Optional[ResourceMetricSource] = Field(
        None,
        description='resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the "pods" source.',
    )
    type: str = Field(
        ...,
        description='type is the type of metric source.  It should be one of "Object", "Pods" or "Resource", each mapping to a matching field in the object.',
    )


class MetricStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    external: Optional[ExternalMetricStatus] = Field(
        None,
        description="external refers to a global metric that is not associated with any Kubernetes object. It allows autoscaling based on information coming from components running outside of cluster (for example length of queue in cloud messaging service, or QPS from loadbalancer running outside of cluster).",
    )
    object: Optional[ObjectMetricStatus] = Field(
        None,
        description="object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object).",
    )
    pods: Optional[PodsMetricStatus] = Field(
        None,
        description="pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value.",
    )
    resource: Optional[ResourceMetricStatus] = Field(
        None,
        description='resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the "pods" source.',
    )
    type: str = Field(
        ...,
        description='type is the type of metric source.  It will be one of "Object", "Pods" or "Resource", each corresponds to a matching field in the object.',
    )


class HorizontalPodAutoscalerSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_replicas: int = Field(
        ...,
        alias="maxReplicas",
        description="maxReplicas is the upper limit for the number of replicas to which the autoscaler can scale up. It cannot be less that minReplicas.",
    )
    metrics: Optional[List[MetricSpec]] = Field(
        None,
        description="metrics contains the specifications for which to use to calculate the desired replica count (the maximum replica count across all metrics will be used).  The desired replica count is calculated multiplying the ratio between the target value and the current value by the current number of pods.  Ergo, metrics used must decrease as the pod count is increased, and vice-versa.  See the individual metric source types for more information about how each type of metric must respond.",
    )
    min_replicas: Optional[int] = Field(
        None,
        alias="minReplicas",
        description="minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one Object or External metric is configured.  Scaling is active as long as at least one metric value is available.",
    )
    scale_target_ref: CrossVersionObjectReference = Field(
        ...,
        alias="scaleTargetRef",
        description="scaleTargetRef points to the target resource to scale, and is used to the pods for which metrics should be collected, as well as to actually change the replica count.",
    )


class HorizontalPodAutoscalerStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    conditions: List[HorizontalPodAutoscalerCondition] = Field(
        ...,
        description="conditions is the set of conditions required for this autoscaler to scale its target, and indicates whether or not those conditions are met.",
    )
    current_metrics: Optional[List[MetricStatus]] = Field(
        None,
        alias="currentMetrics",
        description="currentMetrics is the last read state of the metrics used by this autoscaler.",
    )
    current_replicas: int = Field(
        ...,
        alias="currentReplicas",
        description="currentReplicas is current number of replicas of pods managed by this autoscaler, as last seen by the autoscaler.",
    )
    desired_replicas: int = Field(
        ...,
        alias="desiredReplicas",
        description="desiredReplicas is the desired number of replicas of pods managed by this autoscaler, as last calculated by the autoscaler.",
    )
    last_scale_time: Optional[v1.Time] = Field(
        None,
        alias="lastScaleTime",
        description="lastScaleTime is the last time the HorizontalPodAutoscaler scaled the number of pods, used by the autoscaler to control how often the number of pods is changed.",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="observedGeneration is the most recent generation observed by this autoscaler.",
    )


class HorizontalPodAutoscaler(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "autoscaling/v2beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind53] = Field(
        "HorizontalPodAutoscaler",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="metadata is the standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[HorizontalPodAutoscalerSpec] = Field(
        None,
        description="spec is the specification for the behaviour of the autoscaler. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.",
    )


class HorizontalPodAutoscalerList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "autoscaling/v2beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[HorizontalPodAutoscaler] = Field(
        ..., description="items is the list of horizontal pod autoscaler objects."
    )
    kind: Optional[Kind54] = Field(
        "HorizontalPodAutoscalerList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None, description="metadata is the standard list metadata."
    )



import yaml
import pydantic

# TODO: generate imports
from .gen.io.k8s.api.extensions.v1beta1 import *  # pylint: disable=unused-wildcard-import
from .gen.io.k8s.api.apps.v1 import *  # pylint: disable=unused-wildcard-import
from .gen.io.k8s.api.core.v1 import *  # pylint: disable=unused-wildcard-import
from .gen.io.k8s.apimachinery.pkg.apis.meta.v1 import *  # pylint: disable=unused-wildcard-import

API_VERSION = "1.16.4"


class Chart(list):
    pass


def _parse_object(obj):
    kind = obj["kind"]
    del obj["kind"]  # TODO

    cls = globals()[kind]

    if not issubclass(cls, pydantic.BaseModel):
        raise AttributeError(f"unknown kind {kind}")

    return cls(**obj)


def parse(name):
    docs = yaml.safe_load_all(open(name))
    return [_parse_object(doc) for doc in docs]



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind102, Kind103
from ...apimachinery.pkg.apis.meta import v1
from ..core import v1


class EndpointConditions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ready: Optional[bool] = Field(
        None,
        description="ready indicates that this endpoint is prepared to receive traffic, according to whatever system is managing the endpoint. A nil value indicates an unknown state. In most cases consumers should interpret this unknown state as ready.",
    )


class EndpointPort(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: Optional[str] = Field(
        None,
        description="The name of this port. All ports in an EndpointSlice must have a unique name. If the EndpointSlice is dervied from a Kubernetes service, this corresponds to the Service.ports[].name. Name must either be an empty string or pass IANA_SVC_NAME validation: * must be no more than 15 characters long * may contain only [-a-z0-9] * must contain at least one letter [a-z] * it must not start or end with a hyphen, nor contain adjacent hyphens Default is empty string.",
    )
    port: Optional[int] = Field(
        None,
        description="The port number of the endpoint. If this is not specified, ports are not restricted and must be interpreted in the context of the specific consumer.",
    )
    protocol: Optional[str] = Field(
        None,
        description="The IP protocol for this port. Must be UDP, TCP, or SCTP. Default is TCP.",
    )


class Endpoint(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    addresses: List[str] = Field(
        ...,
        description="addresses of this endpoint. The contents of this field are interpreted according to the corresponding EndpointSlice addressType field. This allows for cases like dual-stack (IPv4 and IPv6) networking. Consumers (e.g. kube-proxy) must handle different types of addresses in the context of their own capabilities. This must contain at least one address but no more than 100.",
    )
    conditions: Optional[EndpointConditions] = Field(
        None,
        description="conditions contains information about the current status of the endpoint.",
    )
    hostname: Optional[str] = Field(
        None,
        description="hostname of this endpoint. This field may be used by consumers of endpoints to distinguish endpoints from each other (e.g. in DNS names). Multiple endpoints which use the same hostname should be considered fungible (e.g. multiple A values in DNS). Must pass DNS Label (RFC 1123) validation.",
    )
    target_ref: Optional[v1.ObjectReference] = Field(
        None,
        alias="targetRef",
        description="targetRef is a reference to a Kubernetes object that represents this endpoint.",
    )
    topology: Optional[Dict[str, Any]] = Field(
        None,
        description="topology contains arbitrary topology information associated with the endpoint. These key/value pairs must conform with the label format. https://kubernetes.io/docs/concepts/overview/working-with-objects/labels Topology may include a maximum of 16 key/value pairs. This includes, but is not limited to the following well known keys: * kubernetes.io/hostname: the value indicates the hostname of the node\n  where the endpoint is located. This should match the corresponding\n  node label.\n* topology.kubernetes.io/zone: the value indicates the zone where the\n  endpoint is located. This should match the corresponding node label.\n* topology.kubernetes.io/region: the value indicates the region where the\n  endpoint is located. This should match the corresponding node label.",
    )


class EndpointSlice(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    address_type: Optional[str] = Field(
        None,
        alias="addressType",
        description="addressType specifies the type of address carried by this EndpointSlice. All addresses in this slice must be the same type. Default is IP",
    )
    api_version: Optional[str] = Field(
        "discovery.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    endpoints: List[Endpoint] = Field(
        ...,
        description="endpoints is a list of unique endpoints in this slice. Each slice may include a maximum of 1000 endpoints.",
    )
    kind: Optional[Kind102] = Field(
        "EndpointSlice",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object's metadata."
    )
    ports: Optional[List[EndpointPort]] = Field(
        None,
        description='ports specifies the list of network ports exposed by each endpoint in this slice. Each port must have a unique name. When ports is empty, it indicates that there are no defined ports. When a port is defined with a nil port value, it indicates "all ports". Each slice may include a maximum of 100 ports.',
    )


class EndpointSliceList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "discovery.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[EndpointSlice] = Field(..., description="List of endpoint slices")
    kind: Optional[Kind103] = Field(
        "EndpointSliceList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(None, description="Standard list metadata.")
